@@ -249,8 +249,9 @@ func (ca *clusterAdmin) CreateTopic(topic string, detail *TopicDetail, validateO
 		return errors.New("you must specify topic details")
 	}
 
-	topicDetails := make(map[string]*TopicDetail)
-	topicDetails[topic] = detail
+	topicDetails := map[string]*TopicDetail{
+		topic: detail,
+	}
 
 	request := NewCreateTopicsRequest(
 		ca.conf.Version,
@@ -366,7 +367,7 @@ func (ca *clusterAdmin) ListTopics() (map[string]TopicDetail, error) {
 		return nil, err
 	}
 
-	topicsDetailsMap := make(map[string]TopicDetail)
+	topicsDetailsMap := make(map[string]TopicDetail, len(metadataResp.Topics))
 
 	var describeConfigsResources []*ConfigResource
 
@@ -375,7 +376,7 @@ func (ca *clusterAdmin) ListTopics() (map[string]TopicDetail, error) {
 			NumPartitions: int32(len(topic.Partitions)),
 		}
 		if len(topic.Partitions) > 0 {
-			topicDetails.ReplicaAssignment = map[int32][]int32{}
+			topicDetails.ReplicaAssignment = make(map[int32][]int32, len(topic.Partitions))
 			for _, partition := range topic.Partitions {
 				topicDetails.ReplicaAssignment[partition.ID] = partition.Replicas
 			}
@@ -439,7 +440,10 @@ func (ca *clusterAdmin) DeleteTopic(topic string) error {
 	}
 
 	// Versions 0, 1, 2, and 3 are the same.
-	if ca.conf.Version.IsAtLeast(V2_1_0_0) {
+	// Version 4 is first flexible version.
+	if ca.conf.Version.IsAtLeast(V2_4_0_0) {
+		request.Version = 4
+	} else if ca.conf.Version.IsAtLeast(V2_1_0_0) {
 		request.Version = 3
 	} else if ca.conf.Version.IsAtLeast(V2_0_0_0) {
 		request.Version = 2
@@ -479,8 +483,12 @@ func (ca *clusterAdmin) CreatePartitions(topic string, count int32, assignment [
 		return ErrInvalidTopic
 	}
 
-	topicPartitions := make(map[string]*TopicPartition)
-	topicPartitions[topic] = &TopicPartition{Count: count, Assignment: assignment}
+	topicPartitions := map[string]*TopicPartition{
+		topic: {
+			Count:      count,
+			Assignment: assignment,
+		},
+	}
 
 	request := &CreatePartitionsRequest{
 		TopicPartitions: topicPartitions,
@@ -615,13 +623,14 @@ func (ca *clusterAdmin) DeleteRecords(topic string, partitionOffsets map[int32]i
 		partitionPerBroker[broker] = append(partitionPerBroker[broker], partition)
 	}
 	for broker, partitions := range partitionPerBroker {
-		topics := make(map[string]*DeleteRecordsRequestTopic)
-		recordsToDelete := make(map[int32]int64)
+		recordsToDelete := make(map[int32]int64, len(partitions))
 		for _, p := range partitions {
 			recordsToDelete[p] = partitionOffsets[p]
 		}
-		topics[topic] = &DeleteRecordsRequestTopic{
-			PartitionOffsets: recordsToDelete,
+		topics := map[string]*DeleteRecordsRequestTopic{
+			topic: {
+				PartitionOffsets: recordsToDelete,
+			},
 		}
 		request := &DeleteRecordsRequest{
 			Topics:  topics,
@@ -785,6 +794,10 @@ func (ca *clusterAdmin) IncrementalAlterConfig(resourceType ConfigResourceType,
 		ValidateOnly: validateOnly,
 	}
 
+	if ca.conf.Version.IsAtLeast(V2_4_0_0) {
+		request.Version = 1
+	}
+
 	var (
 		b   *Broker
 		err error
@@ -972,7 +985,8 @@ func (ca *clusterAdmin) DescribeConsumerGroups(groups []string) (result []*Group
 
 		if ca.conf.Version.IsAtLeast(V2_4_0_0) {
 			// Starting in version 4, the response will include group.instance.id info for members.
-			describeReq.Version = 4
+			// Starting in version 5, the response uses flexible encoding
+			describeReq.Version = 5
 		} else if ca.conf.Version.IsAtLeast(V2_3_0_0) {
 			// Starting in version 3, authorized operations can be requested.
 			describeReq.Version = 3
@@ -1032,10 +1046,7 @@ func (ca *clusterAdmin) ListConsumerGroups() (allGroups map[string]string, err e
 				return
 			}
 
-			groups := make(map[string]string)
-			maps.Copy(groups, response.Groups)
-
-			groupMaps <- groups
+			groupMaps <- maps.Clone(response.Groups)
 		}(b, ca.conf)
 	}
 
@@ -1122,7 +1133,10 @@ func (ca *clusterAdmin) DeleteConsumerGroup(group string) error {
 	request := &DeleteGroupsRequest{
 		Groups: []string{group},
 	}
-	if ca.conf.Version.IsAtLeast(V2_0_0_0) {
+
+	if ca.conf.Version.IsAtLeast(V2_4_0_0) {
+		request.Version = 2
+	} else if ca.conf.Version.IsAtLeast(V2_0_0_0) {
 		request.Version = 1
 	}
 
@@ -1157,10 +1171,12 @@ func (ca *clusterAdmin) DeleteConsumerGroup(group string) error {
 }
 
 func (ca *clusterAdmin) DescribeLogDirs(brokerIds []int32) (allLogDirs map[int32][]DescribeLogDirsResponseDirMetadata, err error) {
-	allLogDirs = make(map[int32][]DescribeLogDirsResponseDirMetadata)
-
+	type result struct {
+		id      int32
+		logdirs []DescribeLogDirsResponseDirMetadata
+	}
 	// Query brokers in parallel, since we may have to query multiple brokers
-	logDirsMaps := make(chan map[int32][]DescribeLogDirsResponseDirMetadata, len(brokerIds))
+	logDirsResults := make(chan result, len(brokerIds))
 	errChan := make(chan error, len(brokerIds))
 	wg := sync.WaitGroup{}
 
@@ -1194,18 +1210,17 @@ func (ca *clusterAdmin) DescribeLogDirs(brokerIds []int32) (allLogDirs map[int32
 				errChan <- response.ErrorCode
 				return
 			}
-			logDirs := make(map[int32][]DescribeLogDirsResponseDirMetadata)
-			logDirs[b.ID()] = response.LogDirs
-			logDirsMaps <- logDirs
+			logDirsResults <- result{id: b.ID(), logdirs: response.LogDirs}
 		}(broker, ca.conf)
 	}
 
 	wg.Wait()
-	close(logDirsMaps)
+	close(logDirsResults)
 	close(errChan)
 
-	for logDirsMap := range logDirsMaps {
-		maps.Copy(allLogDirs, logDirsMap)
+	allLogDirs = make(map[int32][]DescribeLogDirsResponseDirMetadata, len(brokerIds))
+	for logDirsResult := range logDirsResults {
+		allLogDirs[logDirsResult.id] = logDirsResult.logdirs
 	}
 
 	// Intentionally return only the first error for simplicity
@@ -1279,10 +1294,11 @@ func (ca *clusterAdmin) AlterUserScramCredentials(u []AlterUserScramCredentialsU
 // Contains components: strict = false
 // Contains only components: strict = true
 func (ca *clusterAdmin) DescribeClientQuotas(components []QuotaFilterComponent, strict bool) ([]DescribeClientQuotasEntry, error) {
-	request := &DescribeClientQuotasRequest{
-		Components: components,
-		Strict:     strict,
-	}
+	request := NewDescribeClientQuotasRequest(
+		ca.conf.Version,
+		components,
+		strict,
+	)
 
 	b, err := ca.Controller()
 	if err != nil {
