@@ -22,6 +22,25 @@ var (
 	}
 
 	createTopicsRequestV1 = append(createTopicsRequestV0, byte(1))
+
+	createTopicsRequestV5 = []byte{
+		2,
+		6, 't', 'o', 'p', 'i', 'c',
+		255, 255, 255, 255,
+		255, 255,
+		2,          // 1 replica assignment
+		0, 0, 0, 0, // partition index
+		4, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2, // broker ids
+		0, // empty tagged fields
+		2, // 1 config
+		13, 'r', 'e', 't', 'e', 'n', 't', 'i', 'o', 'n', '.', 'm', 's',
+		3, '-', '1',
+		0, // empty tagged fields
+		0, // empty tagged fields
+		0, 0, 0, 100,
+		1,
+		0, // empty tagged fields
+	}
 )
 
 func TestCreateTopicsRequest(t *testing.T) {
@@ -49,4 +68,7 @@ func TestCreateTopicsRequest(t *testing.T) {
 	req.ValidateOnly = true
 
 	testRequest(t, "version 1", req, createTopicsRequestV1)
+
+	req.Version = 5
+	testRequest(t, "version 5", req, createTopicsRequestV5)
 }
