@@ -20,6 +20,15 @@ var (
 		0, 5, 't', 'o', 'p', 'i', 'c',
 		0, 0,
 	}
+
+	deleteTopicsResponseV4 = []byte{
+		0, 0, 0, 100,
+		2,
+		6, 't', 'o', 'p', 'i', 'c',
+		0, 0,
+		0, // empty tagged fields
+		0, // empty tagged fields
+	}
 )
 
 func TestDeleteTopicsResponse(t *testing.T) {
@@ -35,4 +44,7 @@ func TestDeleteTopicsResponse(t *testing.T) {
 	resp.ThrottleTime = 100 * time.Millisecond
 
 	testResponse(t, "version 1", resp, deleteTopicsResponseV1)
+
+	resp.Version = 4
+	testResponse(t, "version 4", resp, deleteTopicsResponseV4)
 }
