@@ -4,8 +4,6 @@ import (
 	"errors"
 	"fmt"
 	"strings"
-
-	"github.com/hashicorp/go-multierror"
 )
 
 // ErrOutOfBrokers is the error returned when the client has run out of brokers to talk to because all of them errored
@@ -88,10 +86,13 @@ var ErrCannotTransitionNilError = errors.New("transaction manager: cannot transi
 // ErrTxnUnableToParseResponse when response is nil
 var ErrTxnUnableToParseResponse = errors.New("transaction manager: unable to parse response")
 
-// MultiErrorFormat specifies the formatter applied to format multierrors. The
-// default implementation is a condensed version of the hashicorp/go-multierror
-// default one
-var MultiErrorFormat multierror.ErrorFormatFunc = func(es []error) string {
+// ErrUnknownMessage when the protocol message key is not recognized
+var ErrUnknownMessage = errors.New("kafka: unknown protocol message key")
+
+// MultiErrorFormat specifies the formatter applied to format multierrors.
+//
+// Deprecated: Please use [errors.Join] instead.
+func MultiErrorFormat(es []error) string {
 	if len(es) == 1 {
 		return es[0].Error()
 	}
@@ -128,15 +129,7 @@ func (err sentinelError) Unwrap() error {
 }
 
 func Wrap(sentinel error, wrapped ...error) sentinelError {
-	return sentinelError{sentinel: sentinel, wrapped: multiError(wrapped...)}
-}
-
-func multiError(wrapped ...error) error {
-	merr := multierror.Append(nil, wrapped...)
-	if MultiErrorFormat != nil {
-		merr.ErrorFormat = MultiErrorFormat
-	}
-	return merr.ErrorOrNil()
+	return sentinelError{sentinel: sentinel, wrapped: errors.Join(wrapped...)}
 }
 
 // PacketEncodingError is returned from a failure while encoding a Kafka packet. This can happen, for example,
