@@ -22,18 +22,15 @@ func (f *FindCoordinatorResponse) decode(pd packetDecoder, version int16) (err e
 	if version >= 1 {
 		f.Version = version
 
-		throttleTime, err := pd.getInt32()
-		if err != nil {
+		if f.ThrottleTime, err = pd.getDurationMs(); err != nil {
 			return err
 		}
-		f.ThrottleTime = time.Duration(throttleTime) * time.Millisecond
 	}
 
-	tmp, err := pd.getInt16()
+	f.Err, err = pd.getKError()
 	if err != nil {
 		return err
 	}
-	f.Err = KError(tmp)
 
 	if version >= 1 {
 		if f.ErrMsg, err = pd.getNullableString(); err != nil {
@@ -57,10 +54,10 @@ func (f *FindCoordinatorResponse) decode(pd packetDecoder, version int16) (err e
 
 func (f *FindCoordinatorResponse) encode(pe packetEncoder) error {
 	if f.Version >= 1 {
-		pe.putInt32(int32(f.ThrottleTime / time.Millisecond))
+		pe.putDurationMs(f.ThrottleTime)
 	}
 
-	pe.putInt16(int16(f.Err))
+	pe.putKError(f.Err)
 
 	if f.Version >= 1 {
 		if err := pe.putNullableString(f.ErrMsg); err != nil {
